package com.huimv.farm.piglet.controller;

import com.huimv.AbstractPlatformController;
import com.huimv.CommonConstant;
import com.huimv.exception.PlatformException;
import com.huimv.farm.piglet.domain.count.DeviceStatusInformation;
import com.huimv.farm.piglet.repo.DeviceStatusInformationRepository;
import com.huimv.response.OkResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * DeviceStatusInformationController
 *
 * @author guojian
 */
@RestController
@RequestMapping(value = "/device-status-information")
public class DeviceStatusInformationController extends AbstractPlatformController {

    @Autowired
    private DeviceStatusInformationRepository objectRepository;

    @RequestMapping("/list")
    public Page<DeviceStatusInformation> list(HttpServletRequest request,
                                              @RequestParam(name = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) int pageNum,
                                              @RequestParam(name = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) int pageSize,
                                              @RequestParam(name = "searchStr", required = false) String searchStr) {
        Pageable pageable = PageRequest.of(pageNum - 1, pageSize, Sort.Direction.DESC, "id");
        if (!StringUtils.isEmpty(searchStr)) {
            return objectRepository.findBySearchStr(searchStr, pageable);
        }
        return objectRepository.findAll(pageable);
    }

    @RequestMapping("/add")
    public DeviceStatusInformation add(HttpServletRequest request,
                                       @RequestParam(name = "") String str,
                                       @RequestParam(name = "") int i) {
        Integer orgId = this.getCurrOrgId(request);
        DeviceStatusInformation obj = new DeviceStatusInformation();

        return this.objectRepository.save(obj);
    }

    @RequestMapping("/update")
    public DeviceStatusInformation update(HttpServletRequest request,
                                          @RequestParam(name = "id") int id,
                                          @RequestParam(name = "") String str,
                                          @RequestParam(name = "status") int i) {
        DeviceStatusInformation obj = this.objectRepository.findById(id).orElse(null);
        if (obj == null) {
            throw new PlatformException("data_not_exist", "更新数据不存在");
        }
        return this.objectRepository.save(obj);
    }

    @RequestMapping("/delete")
    public Object delete(HttpServletRequest httpServletRequest,
                         @RequestParam("id") Integer id) {
        DeviceStatusInformation obj = this.objectRepository.findById(id).orElse(null);
        if (obj == null) {
            throw new PlatformException("data_not_exist", "删除数据不存在");
        }
        this.objectRepository.deleteById(id);
        return new OkResponse("删除成功!");
    }
}
