package com.huimv.farm.piglet.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.huimv.AbstractPlatformController;
import com.huimv.CommonConstant;
import com.huimv.farm.piglet.domain.count.FenceOutPrediction;
import com.huimv.farm.piglet.feign.clients.PublicFeignClient;
import com.huimv.farm.piglet.repo.FenceOutPredictionRepository;
import com.huimv.tools.DateUtils;

/**
 * FenceOutPredictionController
 *
 * @author some one
 */
@RestController
@RequestMapping(value = "/fence-out-prediction")
public class FenceOutPredictionController extends AbstractPlatformController {

    @Autowired
    private FenceOutPredictionRepository objectRepository;
    @Autowired
    private PublicFeignClient publicFeignClient;

    private static Map<String, String> monthName;
    static {
        monthName = new HashMap<>();
        monthName.put("01", "一月");
        monthName.put("02", "二月");
        monthName.put("03", "三月");
        monthName.put("04", "四月");
        monthName.put("05", "五月");
        monthName.put("06", "六月");
        monthName.put("07", "七月");
        monthName.put("08", "八月");
        monthName.put("09", "九月");
        monthName.put("10", "十月");
        monthName.put("11", "十一月");
        monthName.put("12", "十二月");
    }

    @RequestMapping("/list")
    public Page<FenceOutPrediction> list(HttpServletRequest request,
                                         @RequestParam(name = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) int pageNum,
                                         @RequestParam(name = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) int pageSize,
                                         @RequestParam(name = "searchStr", required = false) String searchStr) {
        Pageable pageable = PageRequest.of(pageNum - 1, pageSize, Sort.Direction.DESC, "id");
        if (!StringUtils.isEmpty(searchStr)) {
            return objectRepository.findBySearchStr(searchStr, pageable);
        }
        return objectRepository.findAll(pageable);
    }

    @RequestMapping("/list-area")
    public List<Map<String, Object>> listArea(HttpServletRequest request,
                                              @RequestParam(name = "areaId", required = false, defaultValue = "-1") long areaId) {

        Map<String, Integer> monthCountMap = new HashMap<>();
        List<FenceOutPrediction> fenceOutPredictions;
        //如果为-1则默认是全牧场（多产床）的预测总和：
        if(areaId == -1L){

            //TODO: 加让当前orgId判断
            fenceOutPredictions = objectRepository.findByStatus(0);

        }else {

//            TreeNode treeNode = publicFeignClient.getTreeNode(areaId, false).orElse(null);
//            if (treeNode == null) {
//                throw new PlatformException("area_node_not_found", "未找到该区域节点");
//            }

            fenceOutPredictions = objectRepository.findByStatusAndAreaId(0, areaId);
        }
        for (FenceOutPrediction fp : fenceOutPredictions) {
            Integer count = monthCountMap.get(fp.getMonth());
            if(count == null){
                count = 0;
            }
            monthCountMap.put(fp.getMonth(), count + fp.getPigCount());
        }

        List<String> monthKeys = getPreMonthList();
        List<Map<String, Object>> resultList = new ArrayList<>();
        for (String monthKey : monthKeys) {
            Map<String, Object> item = new HashMap<>();
            item.put("name", monthName.get(monthKey.substring(monthKey.length() - 2)));
            item.put("value", monthCountMap.get(monthKey) == null ? 0 : monthCountMap.get(monthKey));
            resultList.add(item);
        }
        return resultList;
    }

    //当前时间后退6个月的数据：预测（表里只存6个月的数据）
    private List<String> getPreMonthList(){
        List<String> resultList = new ArrayList<>();
        Calendar c = Calendar.getInstance();
        c.setTime(new Date());
        for (int i = 0; i < 6; i++) {
            c.add(Calendar.MONTH, 1);
            resultList.add(DateUtils.formatDate(c.getTime(), DateUtils.DATE_FORMAT_NUMBER_MONTH));
        }
        return resultList;
    }
}
