package com.huimv.farm.piglet.controller;

import com.huimv.CommonConstant;
import com.huimv.exception.PlatformException;
import com.huimv.farm.piglet.common.Constant;
import com.huimv.farm.piglet.domain.ObstetricTable;
import com.huimv.farm.piglet.repo.DeviceRepository;
import com.huimv.farm.piglet.repo.FarrowingHouseRepository;
import com.huimv.farm.piglet.repo.LitterRepository;
import com.huimv.farm.piglet.repo.ObstetricTableRepository;
import com.huimv.tools.DataUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@RestController
@RequestMapping("/table")
public class ObstetricTableController {

    @Autowired
    ObstetricTableRepository obstetricTableRepository;

    @Autowired
    FarrowingHouseRepository farrowingHouseRepository;

    @Autowired
    LitterRepository litterRepository;

    @Autowired
    DeviceRepository deviceRepository;

    @PostMapping("/list")
    public Page<ObstetricTable> list(@RequestParam(value = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) Integer pageNum,
                                          @RequestParam(value = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) Integer pageSize,
                                          String searchStr, Integer building, Integer house) {
        Pageable pageInfo = PageRequest.of(pageNum - 1, pageSize);
        Page<ObstetricTable> tables;
        if (StringUtils.isEmpty(searchStr)) {
            if (house != null) {
                tables = obstetricTableRepository.findByHouseIdOrderByCode(house, pageInfo);
            } else if (building != null) {
                //TODO 重点测试
                tables = obstetricTableRepository.findByHouseIdInOrderByCode(farrowingHouseRepository.findIdByBuildingId(building), pageInfo);
            } else {
                tables = obstetricTableRepository.findByOrderByCode(pageInfo);
            }
        } else {
            tables = obstetricTableRepository.findByCodeContainingOrderByCode(searchStr, pageInfo);
        }

        return tables;
    }

    @PostMapping("/one")
    public ObstetricTable one(@RequestParam(value = "code") String code) {
        Optional<ObstetricTable> table = obstetricTableRepository.findByCode(code);
        if (table.isPresent()) {
            return table.get();
        } else {
            throw new PlatformException("Obstetric Table does not exist", "产床不存在");
        }
    }

    @PostMapping("/device")
    public Page<Map<String, Object>> device(@RequestParam(value = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) Integer pageNum,
                                            @RequestParam(value = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) Integer pageSize,
                                            Integer building, Integer house, String table, Integer status) {
        Pageable pageInfo = PageRequest.of(pageNum - 1, pageSize);
        Page<ObstetricTable> tables;
        if (status == null) {
            if (table != null) {
                Map<String, Object> map = new HashMap<>();
                Optional<ObstetricTable> obstetricTable = obstetricTableRepository.findByCode(table);
                if (obstetricTable.isPresent()) {
                    List<Map<String, Object>> tableList = new ArrayList<>();
                    Optional<ObstetricTable> record = obstetricTableRepository.findByCode(table);
                    map.put("table", record);
                    map.put("device", record.flatMap(value -> deviceRepository.findByCode(value.getDeviceId())).orElse(null));
                    tableList.add(map);
                    return DataUtils.listToPage(tableList, PageRequest.of(CommonConstant.PAGE_NUM - 1, pageSize));
                }else {
                    return DataUtils.listToPage(new ArrayList<>(), PageRequest.of(CommonConstant.PAGE_NUM - 1, pageSize));
                }
            } else {
                if (house != null) {
                    tables = obstetricTableRepository.findByHouseIdOrderByCode(house, pageInfo);
                } else if (building != null) {
                    tables = obstetricTableRepository.findByHouseIdInOrderByCode(farrowingHouseRepository.findIdByBuildingId(building), pageInfo);
                } else {
                    tables = obstetricTableRepository.findByOrderByCode(pageInfo);
                }
                return addDeviceInfo(tables, pageInfo);
            }
        } else {
            tables = obstetricTableRepository.findByStatusOrderByCode(status, pageInfo);
            return addDeviceInfo(tables, pageInfo);
        }
    }

    private Page<Map<String, Object>> addDeviceInfo(Page<ObstetricTable> tables, Pageable pageable) {
        List<ObstetricTable> list = tables.getContent();
        List<Map<String, Object>> res = new ArrayList<>();
        for (ObstetricTable obstetricTable : list) {
            Map<String, Object> oneMap = new HashMap<>();
            oneMap.put("table", obstetricTable);
            if (obstetricTable.getDeviceId() != null) {
                oneMap.put("device", deviceRepository.findByCode(obstetricTable.getDeviceId()));
            } else {
                oneMap.put("device", null);
            }
            res.add(oneMap);
        }
        return DataUtils.listToPage(res, pageable);
    }

    @RequestMapping("/status")
    private Map<String, Object> tableByStatus() {
        Map<String, Object> oneStatus = new HashMap<>();
        oneStatus.put("empty",obstetricTableRepository.countByStatus(Constant.TABLE_STATUS_EMPTY));
        oneStatus.put("entry",obstetricTableRepository.countByStatus(Constant.TABLE_STATUS_ENTRY));
        oneStatus.put("feed",obstetricTableRepository.countByStatus(Constant.TABLE_STATUS_FEED));
        oneStatus.put("weaning",obstetricTableRepository.countByStatus(Constant.TABLE_STATUS_WEANING));

        Integer allDeliver = litterRepository.sumDeliverNumber();
        Integer allWeaning = litterRepository.sumWeaningNumber();
        oneStatus.put("allDeliver", allDeliver);
        oneStatus.put("allWeaning", allWeaning);
        oneStatus.put("weaningRate", allWeaning / (float) litterRepository.sumDeliverNumberAndWeaning());
        oneStatus.put("avgDeliver", (float)(allDeliver / litterRepository.count()));
        oneStatus.put("avgWeaning", (float)(allWeaning / litterRepository.count()));
        oneStatus.put("avgWeaningPeriod", (float)(litterRepository.sumWeaningDate() / litterRepository.count()));
        return oneStatus;
    }
}

