package com.huimv.farm.piglet.controller;

import com.huimv.AbstractPlatformController;
import com.huimv.CommonConstant;
import com.huimv.farm.piglet.domain.count.ProductionTraceability;
import com.huimv.farm.piglet.repo.ProductionTraceabilityRepository;
import com.huimv.tools.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * ProductionTraceabilityController
 *
 * @author some one
 */
@RestController
@RequestMapping(value = "/production-traceability")
public class ProductionTraceabilityController extends AbstractPlatformController {

    @Autowired
    private ProductionTraceabilityRepository objectRepository;

    @RequestMapping("/list")
    public Page<ProductionTraceability> list(HttpServletRequest request,
                                             @RequestParam(name = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) int pageNum,
                                             @RequestParam(name = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) int pageSize,
                                             @RequestParam(name = "searchStr", required = false) String searchStr) {
        Pageable pageable = PageRequest.of(pageNum - 1, pageSize, Sort.Direction.DESC, "id");
        if (!StringUtils.isEmpty(searchStr)) {
            return objectRepository.findBySearchStr(searchStr, pageable);
        }
        return objectRepository.findAll(pageable);
    }

    @RequestMapping("/list-week")
    public List<Map<String, Object>> listWeek(HttpServletRequest request) {
        //当前时间前推五周：
        Date currentWeek = DateUtils.getThisWeekMonday(new Date());
        List<Map<String, Object>> resultList = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            ProductionTraceability pt = objectRepository.findByWeek(DateUtils.formatDate(currentWeek, DateUtils.DATE_FORMAT_NUMBER));
            if(pt == null){
                pt = new ProductionTraceability();
            }
            Map<String, Object> item = new HashMap<>();
            item.put("name", (i + 1) + "周");
            item.put("breedingCount", pt.getBreedingCount());
            item.put("deliveryCount", pt.getDeliveryCount());
            item.put("pregnantCount", pt.getPregnantCount());
            resultList.add(item);
            currentWeek = DateUtils.geLastWeekMonday(currentWeek);
        }
        return resultList;
    }
}
