package com.huimv.farm.piglet.controller;

import com.huimv.AbstractPlatformController;
import com.huimv.CommonConstant;
import com.huimv.farm.piglet.domain.count.PsyAnalysis;
import com.huimv.farm.piglet.domain.count.PsyAnalysisRecord;
import com.huimv.farm.piglet.repo.PsyAnalysisRecordRepository;
import com.huimv.farm.piglet.repo.PsyAnalysisRepository;
import com.huimv.tools.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * PsyAnalysisController
 *
 * @author some one
 */
@RestController
@RequestMapping(value = "/psy-analysis")
public class PsyAnalysisController extends AbstractPlatformController {

    @Autowired
    private PsyAnalysisRepository objectRepository;
    @Autowired
    private PsyAnalysisRecordRepository psyAnalysisRecordRepository;

    private static Map<String, String> monthName;
    static {
        monthName = new HashMap<>();

//        monthName.put("01", "Jan");
//        monthName.put("02", "Feb");
//        monthName.put("03", "Mar");
//        monthName.put("04", "Apr");
//        monthName.put("05", "May");
//        monthName.put("06", "Jun");
//        monthName.put("07", "Jul");
//        monthName.put("08", "Aug");
//        monthName.put("09", "Sep");
//        monthName.put("10", "Oct");
//        monthName.put("11", "Now");
//        monthName.put("12", "Dec");

        monthName.put("01", "一月");
        monthName.put("02", "二月");
        monthName.put("03", "三月");
        monthName.put("04", "四月");
        monthName.put("05", "五月");
        monthName.put("06", "六月");
        monthName.put("07", "七月");
        monthName.put("08", "八月");
        monthName.put("09", "九月");
        monthName.put("10", "十月");
        monthName.put("11", "十一月");
        monthName.put("12", "十二月");
    }

    @RequestMapping("/list")
    public Page<PsyAnalysis> list(HttpServletRequest request,
                                  @RequestParam(name = "pageNum", defaultValue = CommonConstant.PAGE_NUM_STR) int pageNum,
                                  @RequestParam(name = "pageSize", defaultValue = CommonConstant.PAGE_SIZE_STR) int pageSize,
                                  @RequestParam(name = "searchStr", required = false) String searchStr) {
        Pageable pageable = PageRequest.of(pageNum - 1, pageSize, Sort.Direction.DESC, "id");
        if (!StringUtils.isEmpty(searchStr)) {
            return objectRepository.findBySearchStr(searchStr, pageable);
        }
        return objectRepository.findAll(pageable);
    }

    @RequestMapping("/get-data")
    public Map<String, Object> getData(HttpServletRequest request) {
        Map<String, Object> resultMap = new HashMap<>();
        PsyAnalysis psyAnalysis;

        List<PsyAnalysis> psyAnalyses = objectRepository.findAll();
        if(psyAnalyses.size() > 0){
            psyAnalysis = psyAnalyses.get(0);
        }else {
            psyAnalysis = new PsyAnalysis();
        }

        List<Map<String, Object>> resultList = new ArrayList<>();
        List<String> monthList = getPreMonthList();
        for (String monthKey : monthList) {
            PsyAnalysisRecord par = psyAnalysisRecordRepository.findByMonth(monthKey);
            Map<String, Object> item = new HashMap<>();
            item.put("name", monthName.get(monthKey.substring(monthKey.length() - 2)));
            item.put("value", par.getPsyValue());
            resultList.add(item);
        }

        resultMap.put("headData", psyAnalysis);
        //倒叙下，方便前端处理：
        Collections.reverse(resultList);
        resultMap.put("curveData", resultList);
        return resultMap;
    }

    //当前时间后退12个月的数据：PSY曲线
    private List<String> getPreMonthList(){
        List<String> resultList = new ArrayList<>();
        Calendar c = Calendar.getInstance();
        c.setTime(new Date());
        for (int i = 0; i < 12; i++) {
            c.add(Calendar.MONTH, -1);
            resultList.add(DateUtils.formatDate(c.getTime(), DateUtils.DATE_FORMAT_NUMBER_MONTH));
        }
        return resultList;
    }
}
