package com.huimv.farm.piglet.mq;

import java.util.HashMap;
import java.util.Map;

import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.huimv.analyse.domain.ear_tag.EarTagReport;
import com.huimv.farm.piglet.domain.Sow;
import com.huimv.farm.piglet.repo.SowRepository;
import com.huimv.mq.MqConstant;

import lombok.extern.slf4j.Slf4j;

/**
 * @author yejijie
 * @created 2020年8月25日 上午9:02:31
*/
@Slf4j
@Component
public class EarTagListener {

	@Autowired
	private SowRepository sowRepository;
	
    @RabbitListener(
    		bindings = @QueueBinding(
    				value = @Queue(value = MqConstant.QUEUE_EAR_TAG_PIGLET,autoDelete = "true",declare = "true"),
    				exchange = @Exchange(name=MqConstant.EXCHANGE_EAR_TAG_TOPIC,type = ExchangeTypes.TOPIC),
    				key = MqConstant.KEY_EAR_TAG_REPORT_FANOUT))
    @RabbitHandler
    private void recvErTagData(EarTagReport rep) {
    	log.info("device->eartag->"+rep.toString());
    	if(rep.getDeviceId() != null) {//理论上不可能出现
        	Sow sow = this.sowRepository.findByEarTagIdAndIsEnable(rep.getDeviceId(),true).orElse(null);
        	if(sow != null) {
        		Map<String,Object> data = sow.getData();
        		if(data == null) {
        			data = new HashMap<>();
        			sow.setData(data);
        		}
        		data.put("addr", rep.getAddr());
        		data.put("animalTemp", rep.getAnimalTemp());
        		data.put("envTemp", rep.getEnvTemp());
        		data.put("battery", rep.getBattery());
        		this.sowRepository.save(sow);
        	}
    	}
    }
}
