package com.huimv.farm.piglet.mq;

import java.util.HashMap;
import java.util.Map;

import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.huimv.analyse.domain.piggy.PiggyData;
import com.huimv.farm.piglet.domain.ObstetricTable;
import com.huimv.farm.piglet.repo.LitterRepository;
import com.huimv.farm.piglet.repo.ObstetricTableRepository;
import com.huimv.mq.MqConstant;

import lombok.extern.slf4j.Slf4j;

/**
 * @author yejijie
 * @created 2020年8月22日 下午5:12:40
*/
@Slf4j
@Component
public class PiggyDataListener {

	@Autowired
	private ObstetricTableRepository obstetricTableRepository;
	@Autowired
	private LitterRepository litterRepository;
	
	@RabbitListener(
	    		bindings = @QueueBinding(
	    				value = @Queue(value = MqConstant.QUEUE_PIGGY_DATA_PIGLET,autoDelete = "true",declare = "true"),
					exchange = @Exchange(name=MqConstant.EXCHANGE_PIGGY_TOPIC,type = ExchangeTypes.TOPIC),
					key = MqConstant.PIGGY_DATA_FANOUT_ROUTING_KEY))
	@RabbitHandler
	private void recvPiggyData(PiggyData pd) {
		if(pd.getId()>0) {
			log.info("收到小猪设备信息->"+pd.getSn());
			ObstetricTable table = this.obstetricTableRepository.findByDeviceIdAndDeviceCh(pd.getDeviceId(), pd.getCh()).orElse(null);
			if(table != null) {

				Map<String,Object> data = new HashMap<>();
				//int ch = pd.getCh();//这是通道，代表床编号，应用层会保存通道号在产床配置里
				data.put("file1", pd.getFile1());
				data.put("file2", pd.getFile2());
				data.put("piggy_num", pd.getPiggyNum());
				data.put("piggy_list", pd.getPiggyList());
				data.put("temp", pd.getTemp());
				
				if(table.getData() == null) 
					table.setData(data);
				else
					table.getData().putAll(data);
				this.obstetricTableRepository.save(table);
				if(table.getLitter()!=null) {
					table.getLitter().setDeliverNumber(pd.getPiggyNum());
					this.litterRepository.save(table.getLitter());
				}
			}
		}
	}
}
