package com.huimv.farm.piglet.repo;

import com.huimv.farm.piglet.domain.Litter;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;

import java.util.Date;
import java.util.List;

public interface LitterRepository extends JpaRepository<Litter, Integer> {

    @Query(nativeQuery = true, value = "SELECT SUM(deliver_number) FROM litter")
    Integer sumDeliverNumber();

    @Query(nativeQuery = true, value = "SELECT SUM(deliver_number) FROM litter WHERE weaning_ IS NOT NULL ")
    Integer sumDeliverNumberAndWeaning();

    @Query(nativeQuery = true, value = "SELECT SUM(weaning_number) FROM litter")
    Integer sumWeaningNumber();

    @Query(nativeQuery = true, value = "SELECT SUM(DATEDIFF(weaning_time, deliver_time)) FROM litter ")
    Integer sumWeaningDate();

    @Query(nativeQuery = true, value = "SELECT mother_id FROM litter WHERE weaning_time IS NOT NULL GROUP BY mother_id HAVING COUNT(id) > :parity")
    List<Integer> cullingParity(Integer parity);

    @Query(nativeQuery = true, value = "SELECT mother_id FROM litter WHERE deliver_number < :num AND weaning_time IS NOT NULL GROUP BY mother_id HAVING COUNT(id) > :parity")
    List<Integer> cullingDeliverNumTotal(Integer num,Integer parity);

    //正在哺乳的猪
    @Query(nativeQuery = true, value = "SELECT mother_id FROM litter WHERE deliver_time IS NOT NULL AND weaning_time IS NULL")
    List<Integer> findMotherIdByDeliverTimeIsNotNullAndWeaningTimeIsNull();

    @Query(nativeQuery = true, value = "SELECT distinct(mother_id) FROM litter WHERE weaning_time IS NOT NULL AND mother_id NOT IN :exclude")
    List<Integer> notInWeaning(List<Integer> exclude);

    @Query(nativeQuery = true, value = "SELECT * FROM litter WHERE mother_id=:mother ORDER BY deliver_time DESC limit 2")
    List<Litter> lastTwoRecord(Integer mother);

    @Query(nativeQuery = true, value = "SELECT mother_id FROM litter WHERE (weaning_number/deliver_number)*100 < :rate AND weaning_time IS NOT NULL GROUP BY mother_id HAVING COUNT(id) > :parity")
    List<Integer> cullingDeliverSurviveTotal(Integer rate,Integer parity);

    @Query(nativeQuery = true, value = "SELECT * FROM litter WHERE deliver_time IS NULL AND DATEDIFF(due_date,now()) = :period")
    List<Litter> periodToDue(Integer period);

    @Query(nativeQuery = true, value = "SELECT * FROM litter WHERE weaning_time IS NULL AND DATEDIFF(now(), deliver_time) = :period")
    List<Litter> periodAfterDeliver(Integer period);

    @Query(nativeQuery = true, value = "SELECT * FROM litter WHERE DATEDIFF(now(), weaning_time) = :period")
    List<Litter> periodAfterWeaning(Integer period);

    @Query(nativeQuery = true, value = "SELECT SUM(deliver_number) FROM litter WHERE deliver_time > :startTime AND deliver_time < :endTime")
    Integer sumDeliverNumberInPeriod(Date startTime, Date endTime);

    @Query(nativeQuery = true, value = "SELECT COUNT(deliver_number) FROM litter WHERE deliver_time > :startTime AND deliver_time < :endTime")
    Integer countDeliverNumberInPeriod(Date startTime, Date endTime);

    @Query(nativeQuery = true, value = "SELECT distinct(mother_id) FROM litter WHERE deliver_time BETWEEN :startTime AND :endTime HAVING COUNT(id) > 1")
    List<Integer> findByDeliverTimeBetween(Date startTime, Date endTime);

    @Query(nativeQuery = true, value = "SELECT * FROM litter WHERE mother_id = :motherId AND deliver_time BETWEEN :startTime AND :endTime ORDER BY deliver_time DESC")
    List<Litter> findByDeliverInPeriod(Integer motherId, Date startTime, Date endTime);
}
