package com.huimv.farm.piglet.task;

import com.huimv.exception.PlatformException;
import com.huimv.farm.piglet.domain.Litter;
import com.huimv.farm.piglet.domain.count.PsyAnalysis;
import com.huimv.farm.piglet.domain.count.PsyAnalysisRecord;
import com.huimv.farm.piglet.repo.LitterRepository;
import com.huimv.farm.piglet.repo.PsyAnalysisRecordRepository;
import com.huimv.farm.piglet.repo.PsyAnalysisRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Component
@SuppressWarnings("unused")
@Slf4j
public class PsyTask {

    @Autowired
    PsyAnalysisRecordRepository psyAnalysisRecordRepository;

    @Autowired
    LitterRepository litterRepository;

    @Autowired
    PsyAnalysisRepository psyAnalysisRepository;

    //每月一次计算PSY
    //@Scheduled(cron = "0 30 1 1 * ?")
    @Scheduled(cron = "* */1 * * * ?")
    private void monthPsy() {
//        System.out.println("开始");
        SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat month = new SimpleDateFormat("yyyyMM");
        Calendar today = Calendar.getInstance();
        String monthStart = today.get(Calendar.YEAR) + "-" + (today.get(Calendar.MONTH) + 1) + "-01 00:00:00";
        today.add(Calendar.MONTH, -1);
        String lastMonthStart = today.get(Calendar.YEAR) + "-" + (today.get(Calendar.MONTH) + 1) + "-01 00:00:00";

        int monthDeliver;
        Date monthStartDate;
        Date monthEndDate;
        try {
            monthStartDate = dateTime.parse(lastMonthStart);
            monthEndDate = dateTime.parse(monthStart);
        } catch (Exception e) {
            throw new PlatformException("method monthPsy() time format failed", "方法monthPsy()时间转换错误");
        }

        monthDeliver = litterRepository.sumDeliverNumberInPeriod(monthStartDate, monthEndDate);
        //PSY分析-PSY
        Double psy = (monthDeliver * 12) / Double.valueOf(litterRepository.countDeliverNumberInPeriod(monthStartDate, monthEndDate));

        PsyAnalysisRecord psyAnalysisRecord = new PsyAnalysisRecord();
        psyAnalysisRecord.setMonth(month.format(monthEndDate));
        psyAnalysisRecord.setPsyValue(psy);
        psyAnalysisRecordRepository.save(psyAnalysisRecord);

        //月环比
        today.add(Calendar.MONTH, -1);
        int tempMonth = today.get(Calendar.MONTH) + 1;
        String lastTwoMonth = today.get(Calendar.YEAR) + tempMonth < 10 ? "0" + tempMonth : tempMonth + "";
        //年同比
        today.add(Calendar.MONTH, 1);
        today.add(Calendar.YEAR, -1);
        int tempMonth2 = today.get(Calendar.MONTH) + 1;
        String lastYear = today.get(Calendar.YEAR) + tempMonth2 < 10 ? "0" + tempMonth2 : tempMonth2 + "";

        //PSY分析-PSY年同比
        PsyAnalysisRecord psyYear = psyAnalysisRecordRepository.findByMonth(lastYear);
        //PSY分析-PSY月环比
        PsyAnalysisRecord psyTwoMonth = psyAnalysisRecordRepository.findByMonth(lastTwoMonth);

        Date lastYearStartDate;
        Date thisYearStartDate;
        Date nextYearStartDate;
        try {
            String lastYearStart = today.get(Calendar.YEAR) + "-01-01 00:00:00";
            lastYearStartDate = dateTime.parse(lastYearStart);
            today.add(Calendar.YEAR, 1);
            String thisYearStart = today.get(Calendar.YEAR) + "-01-01 00:00:00";
            thisYearStartDate = dateTime.parse(thisYearStart);
            today.add(Calendar.YEAR, 1);
            String nextYearStart = today.get(Calendar.YEAR) + "-01-01 00:00:00";
            nextYearStartDate = dateTime.parse(nextYearStart);
        } catch (Exception e) {
            throw new PlatformException("method monthPsy() time format failed", "方法monthPsy()时间转换错误");
        }

        Double lastAvgDeliver = getAvgDeliver(lastYearStartDate, thisYearStartDate);
        Double thisAvgDeliver = getAvgDeliver(thisYearStartDate, nextYearStartDate);
        //PSY分析-非生产天数
        double thisNoDeliverPeriod = thisAvgDeliver - 114;
        //PSY分析-年同比
        double noDeliverRate =  lastAvgDeliver == 0 ? 0 : (thisAvgDeliver - lastAvgDeliver) / lastAvgDeliver;
        //PSY分析-平均胎次
        double avgParity = lastAvgDeliver == 0 ? 0 : (thisAvgDeliver - lastAvgDeliver) * 365 / (lastAvgDeliver * lastAvgDeliver * thisAvgDeliver) ;
        //PSY分析-平均胎次年同比
        double avgParityRate = 365 / thisAvgDeliver ;

        List<PsyAnalysis> list = psyAnalysisRepository.findAll();
        if (list.size() > 0) {
            PsyAnalysis psyAnalysis = list.get(0);
            psyAnalysis.setPsyValue(psy);
            psyAnalysis.setPsyAnnualRatio(psyYear == null ? 0 : psyYear.getPsyValue());
            psyAnalysis.setPsyMonthlyRatio(psyTwoMonth == null ? 0 : psyTwoMonth.getPsyValue());
            psyAnalysis.setLsyValue(new BigDecimal(thisNoDeliverPeriod).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            psyAnalysis.setLsyAnnualRatio(new BigDecimal(noDeliverRate).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            psyAnalysis.setFetusValue(new BigDecimal(avgParity).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            psyAnalysis.setFetusAnnualRatio(new BigDecimal(avgParityRate).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            psyAnalysisRepository.save(psyAnalysis);
        }
//        System.out.println("结束");
    }

    private Double getAvgDeliver(Date lastYearStartDate, Date thisYearStartDate){
        List<Integer> thisYearDeliver = litterRepository.findByDeliverTimeBetween(lastYearStartDate, thisYearStartDate);
        List<Integer> twoDeliverDays = new ArrayList<>();

        for (Integer motherId : thisYearDeliver) {
            List<Litter> deliverInPeriod = litterRepository.findByDeliverInPeriod(motherId, lastYearStartDate, thisYearStartDate);
            for (int i = 0; i < deliverInPeriod.size() - 1; i++) {
                Long days = (deliverInPeriod.get(0).getDeliverTime().getTime() - deliverInPeriod.get(1).getDeliverTime().getTime()) / (20 * 60 * 60 * 1000);
                twoDeliverDays.add(days.intValue());
            }
        }

        int allDeliver = 0;
        for (Integer oneRecord : twoDeliverDays){
            allDeliver += oneRecord;
        }
        return twoDeliverDays.size() == 0 ? 0 : allDeliver / (double)twoDeliverDays.size();
    }
}
